//--------------------------------------------------------------------------------------------------
//
// Title       : ALU
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Arithmetic logic unit
//					Performs combinational operations on two inputs. Input A is hardwired,
//					Input B can be selected either as Register B or Immediate according to srcB
//					signal.
//
//					Operation is defined through ALUop bus according to case statement
//					in Veriolog code.
//--------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps

module ALU(
	input	[31:0]	A,
	input	[31:0]	RegB,
	input	[31:0]	Imm,
	input			srcB,
	input	[5:0]	ALUop,
	output	[31:0]	Out,
	output			zero,
	output			neg
);

// Define outputs as 'reg', since they
//are set in an always block
reg  [31:0] Out;
reg  zero;
reg  neg;
 
// use srcB to select between RegB and Imm
wire [31:0]	B = (srcB == 0)? RegB : Imm;
  
// perform the ALU operation and set flags if the result is zero or negative
always @(A or B or ALUop) begin
	case (ALUop)
		6'b000001: Out = A + B;						// Add
		6'b000010: Out = A - B;						// Substract
		6'b000100: Out = A & B;						// logical And
		6'b001000: Out = A | B;						// logical Or
		6'b010000: Out = (A < B)? 1 : 0;  			// Set on less than. Enables all branches using BEQ
		6'b000011: Out = {B[15:0], 16'h0000}; 		// LUI - Put the immediate into the 16 high order bits, 0 out lower bits
		6'b000101: Out = A | {16'h0000, B[15:0]};	// ORI - Zero extend immed, OR with A
		default:   Out = 32'hxxxxxxxx;
	endcase
	
	zero = (Out == 32'h00000000);					// Test for zero
	neg  = Out[31];    								// Test for negative
end

endmodule
