//--------------------------------------------------------------------------------------------------
//
// Title       : DataMemroy
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Data memory. Has space for 256 32 Bit values
//
//					Reads are performed when readData is asserted, otherwise read output
//					is tri-stated (disconnected from bus)
//
//					Writes are performed on rising edges of writeData
//
//					Module provides memory mapped I/O through writing to address
//					0xffffffff with the pattern (command, value), where command
//					is the high 16 Bits of writeData and value is the low 16 Bits
//
//					command = 16'h0001 - Store value as Disc #
//					command = 16'h0002 - Store value as 'from tower' #
//					command = 16'h0003 - Output disc move with value as 'to tower' #
//					command = 16'h0004 - Debug output value
//--------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps
module DataMemory(
	input	[31:0]	address,
	input			write,
	input			read,
	output	[31:0]	readData,
	input	[31:0]	writeData
);


// slow down the write signal so that we have some setup time for the data coming in
wire		delayed_write;
assign	#10	delayed_write = write;
  
// this is the actual memory array 
reg [31:0] memory[0:255];

// Initialize memory
// input for our programs is stored in location 254
// (0xfe), output will be in 255 (0xff)
integer i;
initial begin	   
	for( i = 0; i < 256; i=i+1) begin
		memory[i] = 32'h00000000;
	end
	memory[8'hfe] = 32'h00000003;			        // this is the input N
end

// IO Buffer for memory mapped IO
reg [1:0] ioBuffer[15:0];


// the rising edge of the delayed_write control input causes data to be written into memory 
// using delayed_write ensures some setup time, otherwise we'd be dependent on the order 
// in which write and data change - if write happened first, we'd write the wrong value
always @(posedge delayed_write) begin
	//Allow for writing to console if a certain memory location is referenced
	if(address === 32'hffffffff) begin
		case(writeData[31:16])
			16'h0001: begin
				ioBuffer[0] = writeData[15:0]; // store disc #
			end
			16'h0002: begin
				ioBuffer[1] = writeData[15:0]; // store value of "from" peg
			end
			16'h0003: begin
				$display("Move Disc %d from peg %d to peg %d",ioBuffer[0],ioBuffer[1],writeData[15:0]);
			end
			16'h0004: begin
				$display("Debug print, value: %d (%h)",writeData[15:0],writeData[15:0]);
			end
			default:
				$display("Write sequence invalid!");
		endcase
	end

	//Otherwise, just do the normal write
	else begin
		// Uncomment to debug memory writes
		//$display("address=%h; data=%h; write=%h", address[7:0], writeData, write);
		memory[address[7:0]] = writeData;
	end
end

// data is always read if read is asserted, otherwise the data bus is tristated
assign readData = read ? memory[address[7:0]] : 32'hzzzzzzzz;

endmodule
