//--------------------------------------------------------------------------------------------------
//
// Title       : InstMemory	(Towers of Hanoi)
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Instruction memory that is preloaded with a program to solve
//					the Towers of Hanoi
//
//-------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps
module InstMemory(address, Inst);

  // parameters so that it is easy to build up the instructions we'll put in the memory
  
  parameter ALU		= 6'h00; // op =  0
  parameter LW		= 6'h23; // op = 35
  parameter SW		= 6'h2b; // op = 43
  parameter BEQ		= 6'h04; // op =  4 
  parameter ADDI	= 6'h08; // op =  8
  parameter J		= 6'h02; // op =  2
  parameter HALT	= 6'h3f; // op = 63
  
  //Added stuff
  parameter LUI		= 6'h10; // op = 16
  parameter ORI		= 6'h0d; // op = 13
  parameter	JAL		= 6'h03; // op =  3
  
  parameter shftX 	= 5'hxx; // we won't be using shift amount
  
  
  // register aliases used in MIPS assembly
  parameter rz 		= 5'h00; // register 0 stores a zero
  parameter v0 		= 5'h02; // used for function result
  parameter v1		= 5'h03; // "                      "
  parameter a0      = 5'h04; // argument 0 for calls
  parameter a1		= 5'h05;
  parameter a2		= 5'h06;
  parameter a3      = 5'h07; 
  parameter t0      = 5'h08; // temps 0-7 
  parameter t1		= 5'h09; // by convention these aren't saved
  parameter t2		= 5'h0A; // across procedure calls
  parameter t3 		= 5'h0B;
  parameter t4		= 5'h0C;
  parameter t5		= 5'h0D;
  parameter t6		= 5'h0E;
  parameter t7		= 5'h0F;
  parameter s0      = 5'h10; // saved 0-7
  parameter s1		= 5'h11; // by convention any code modifying these registers
  parameter s2		= 5'h12; // must save the current values and restore the values
  parameter s3      = 5'h13; // before returning from the function
  parameter s4		= 5'h14;  
  parameter s5		= 5'h15; 
  parameter s6		= 5'h16;
  parameter s7		= 5'h17;
  parameter sp      = 5'h1D; // stack pointer
  parameter fp      = 5'h1E; // frame pointer
  parameter ra      = 5'h1F; // return address


  parameter ADD 	= 6'h20; // funct = 32
  parameter SUB 	= 6'h22; // funct = 34
  parameter AND 	= 6'h24; // funct = 36
  parameter OR 		= 6'h25; // funct = 37
  parameter SLT 	= 6'h2a; // funct = 42
  parameter JR		= 6'h08; // funct =  8

  input	[31:0] address;
  output [31:0] Inst;
   
  // this is the actual memory array 
  
  reg [31:0] memory[0:255];

  // initialize memory with a program to solve the towers of hanoi. the program 
  // reads the number of discs from location 254 of data memory. Then it uses recursive
  // calls to determine the sequence of events, and communicates these by storing specially
  // constructed words to data address 32'hffffffff. This is called memory-mapped i/o. Here's the
  // format(see dmemory.v for more info):
  //     bits 32:16  action
  //       16'h0001  printf("Move disc %d",bits[15:0]);
  //	   16'h0002  printf(" from peg %d",bits[15:0]);
  //       16,h0003  printf(" to peg %d\n",bits[15:0]);
  //

  integer i;
  
  initial begin									  
	// // save information from caller 
	memory[8'h00] = {ADDI,sp ,sp ,      16'hfff8};   //#stack frame is 32 bytes long
	memory[8'h01] = {SW  ,sp ,ra ,      16'h0005};	
	memory[8'h02] = {SW  ,sp ,fp ,      16'h0004};
	memory[8'h03] = {ADDI,sp ,fp ,      16'h0000};
	// // get number of discs from memory location 254 
	memory[8'h04] = {LW  ,rz, a0 ,      16'h00fe};  //# num_pegs = 1st arg
	memory[8'h05] = {ADDI,rz, a1 ,      16'h0001}; //# 1 = 2nd arg
	memory[8'h06] = {ADDI,rz, a2 ,      16'h0002}; //# 2 = 3rd arg
	memory[8'h07] = {ADDI,rz ,a3 ,      16'h0003}; //# 3 = 4th arg
	// hanoi( n , 1 , 2, 3) // move discs n..0 from 1 to 2 using 3 as temp
	memory[8'h08] = {JAL ,           26'h000000A}; // # call procedure
	memory[8'h09] = {HALT,           26'hxxxxxxx}; // exit:	
	// int hanoi(int n, int A, int B, int Temp){
	//   if(n == 0) { return 0; } 
	memory[8'h0A] = {BEQ ,a0 ,rz ,      16'h002A}; // hanoi: #return if n = 1 
    //   make room to store n, A, B, Temp on stack 
	memory[8'h0B] = {ADDI,sp ,sp ,      16'hfffb}; //#adjust stack pointer for args, ra
	memory[8'h0C] = {SW  ,sp ,ra ,      16'h0004}; //#push return address
	memory[8'h0D] = {SW  ,sp ,a3 ,      16'h0003}; //#push Temp
	memory[8'h0E] = {SW  ,sp ,a2 ,      16'h0002}; //#push B
	memory[8'h0F] = {SW  ,sp ,a1 ,      16'h0001}; //#push A
	memory[8'h10] = {SW  ,sp ,a0 ,      16'h0000}; //#push n
	//   make local copies of n, A, B 
	memory[8'h11] = {ALU ,a0, rz, t5 ,shftX, ADD}; //# store local copy of a0
	memory[8'h12] = {ALU ,a1 ,rz ,t6 ,shftX, ADD}; //# "    "
	memory[8'h13] = {ALU ,a2 ,rz ,t7 ,shftX, ADD}; //# "    " 
	//  setup args for recursive call 
	memory[8'h14] = {ADDI,t5 ,a0 ,      16'hffff}; //# n-1 = arg1
	memory[8'h15] = {LW  ,sp ,a1 ,      16'h0001}; //# arg2 = A from caller
	memory[8'h16] = {LW  ,sp ,a2 ,      16'h0003}; //# arg3 = Temp from caller
	memory[8'h17] = {LW  ,sp ,a3 ,      16'h0002}; //# arg4 = B from caller	
	//  hanoi(n-1, A, Temp, B) 
	memory[8'h18] = {JAL ,           26'h000000A}; //# recursive call
	//  get old values of n, A, B, Temp off stack
	memory[8'h19] = {LW  ,sp ,a0 ,      16'h0000}; //# restore a0 = n 
	memory[8'h1A] = {LW  ,sp ,a1 ,      16'h0001}; //# restore a1 = A
	memory[8'h1B] = {LW  ,sp ,a2 ,      16'h0002}; //# restore a2 = B
	memory[8'h1C] = {LW  ,sp ,a3 ,      16'h0003}; //# restore a3 = Temp
	//  make local copies
	memory[8'h1D] = {ADDI,a0 ,t5 ,      16'h0000}; //		
	memory[8'h1E] = {ADDI,a1 ,t6 ,      16'h0000}; //
	memory[8'h1F] = {ADDI,a2 ,t7 ,      16'h0000}; //
	// printf("Move disc %d ",n); 
	memory[8'h20] = {ADDI,rz ,a0 ,      16'hffff}; // # set address of i/o write port
	memory[8'h21] = {LUI ,rz ,t1 ,      16'h0001}; // # set command to write "Move disc %d "
	memory[8'h22] = {ALU ,t5 ,t1 ,t1 ,shftX,  OR}; // # set lower bits to column in t5
	memory[8'h23] = {SW  ,a0 ,t1 ,      16'h0000}; // # call write 
	// printf(" from peg %d",A);
	memory[8'h24] = {LUI ,rz ,t1 ,      16'h0002}; // # set command to write "from peg %d "
	memory[8'h25] = {ALU ,t6 ,t1 ,t1 ,shftX,  OR}; // # set lower bits to column in t6
	memory[8'h26] = {SW  ,a0 ,t1 ,      16'h0000}; // # call write 
	// printf(" to peg %d\n",B);
	memory[8'h27] = {LUI ,rz ,t1 ,      16'h0003}; // # set command to write "to peg %d "
	memory[8'h28] = {ALU ,t7 ,t1 ,t1 ,shftX,  OR}; // # set lower bits to column in t7 
	memory[8'h29] = {SW  ,a0 ,t1 ,      16'h0000}; // # call write
	// setup args for recursive call
	memory[8'h2A] = {ADDI,t5 ,a0 ,      16'hffff}; // # n-1 = 1st arg
	memory[8'h2B] = {LW  ,sp ,a1 ,  	16'h0003}; // # arg2 = Temp from caller
	memory[8'h2C] = {LW  ,sp ,a2 ,  	16'h0002}; // # arg3 = B from caller
	memory[8'h2D] = {LW  ,sp ,a3 ,  	16'h0001}; // # arg4 = A from caller
	// hanoi(n-1,Temp,B,A); 
	memory[8'h2E] = {JAL ,           26'h000000A}; // # second recursive call
	// get ready to return from subroutine
	memory[8'h2F] = {LW  ,sp ,ra ,      16'h0004}; // #restore return address
	memory[8'h30] = {LW  ,sp ,a3 ,      16'h0003}; // #restore a3
	memory[8'h31] = {LW  ,sp ,a2 ,      16'h0002}; // #restore a2
	memory[8'h32] = {LW  ,sp ,a1 ,      16'h0001}; // #restore a1
	memory[8'h33] = {LW  ,sp ,a0 ,      16'h0000}; // #restore a0
	memory[8'h34] = {ADDI,sp ,sp ,      16'h0005}; // #pop stack (sp = sp + 5)
	// return 0;
	// }
	memory[8'h35] = {ALU ,ra ,rz ,rz ,shftX,  JR}; // #base: return from subroutine
	for(i = 8'h36; i <= 8'hff; i = i + 1) begin
		memory[i] = 32'hxzxzxzxz;  // fill rest of memory with invalid stuff
	end
  end

  // data is always read if read is asserted, otherwise the data bus is tristated
	  
  assign Inst = memory[address[7:0]];

endmodule
