//--------------------------------------------------------------------------------------------------
//
// Title       : RegFile
// Design      : mips2000
//
//-------------------------------------------------------------------------------------------------
//
// Description :	Register file, provides 32 general purpose registers
//					Two parallel asynchronous read inputs are provided, the registers
//					to read from are directly extracted from the instruction word
//
//					One register can be synchronously written per clock cycle
//					To write the controller has to assert regWrite and set the
//					wrRegSel bus according to the target register
//
// wrRegSel:
// 00 - write to register supplied as rd
// 01 - write to register supplied as rt
// 1X - Write to register 0x1F (31), used for JAL instruction
//--------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps

module RegFile(
	input	[31:0]	Inst,			// Grab register addresses from instruction word
	output	[31:0]	RegA,			// Output for read A
	output	[31:0]	RegB,			// Output for read B

	input			regWrite,		// Indicate if we want to write
	input	[31:0]	WriteData,		// If so, this is the data to be written
	input	[1:0]	wrRegSel,		// Informs us where to find the target register for write
	
	input	clk
);


// 32 Array of 32-Bit registers
reg [31:0]	RegFile[0:31];

// Initialize register 0 to be a Zero - We'll use this as a constant
// Set stack and frame pointer to initial values and
// return address to something (hopefully) invalid
initial begin
	RegFile[5'h00] = 0;	
	RegFile[5'h1D] = 32'hFFFFFFF0; // set the initial stack pointer
	RegFile[5'h1E] = 32'hFFFFFFF0; // set the initial frame pointer
	RegFile[5'h1F] = 32'hDEADBEEF; // set the initial return address
end


// Aliases:

// Pick out the indices of the three registers possibly
// involved in an instruction 
wire [4:0]	rs	= Inst[25:21];
wire [4:0]	rt	= Inst[20:16];
wire [4:0]	rd	= Inst[15:11];
  
// Decide which register is the one that might be written to,
// depending on select received from controller
wire [4:0]	wrReg = (wrRegSel[1])	? 5'h1F
									: (wrRegSel[0] ? rt : rd);

// Do the two reads
assign 	RegA = RegFile[rs];
assign	RegB = RegFile[rt];
    
// Write into a register when instructed to
// (but never overwrite the register storing our constant 0)	
always @(posedge clk) begin
	if (regWrite && (wrReg != 5'h00)) begin
		RegFile[wrReg] <= WriteData;
	end
end

endmodule
